# Generated by cloudcoil-model-codegen v0.0.17
# DO NOT EDIT

from __future__ import annotations

from typing import Annotated, List, Literal, Optional

from pydantic import Field

from cloudcoil import apimachinery
from cloudcoil._pydantic import BaseModel
from cloudcoil.resources import Resource, ResourceList

from ..core import v1


class StatefulSetOrdinals(BaseModel):
    start: Annotated[
        Optional[int],
        Field(
            description="start is the number representing the first replica's index. It may be used to number replicas from an alternate index (eg: 1-indexed) over the default 0-indexed names, or to orchestrate progressive movement of replicas from one StatefulSet to another. If set, replica indices will be in the range:\n  [.spec.ordinals.start, .spec.ordinals.start + .spec.replicas).\nIf unset, defaults to 0. Replica indices will be in the range:\n  [0, .spec.replicas)."
        ),
    ] = None


class StatefulSetPersistentVolumeClaimRetentionPolicy(BaseModel):
    when_deleted: Annotated[
        Optional[str],
        Field(
            alias="whenDeleted",
            description="WhenDeleted specifies what happens to PVCs created from StatefulSet VolumeClaimTemplates when the StatefulSet is deleted. The default policy of `Retain` causes PVCs to not be affected by StatefulSet deletion. The `Delete` policy causes those PVCs to be deleted.",
        ),
    ] = None
    when_scaled: Annotated[
        Optional[str],
        Field(
            alias="whenScaled",
            description="WhenScaled specifies what happens to PVCs created from StatefulSet VolumeClaimTemplates when the StatefulSet is scaled down. The default policy of `Retain` causes PVCs to not be affected by a scaledown. The `Delete` policy causes the associated PVCs for any excess pods above the replica count to be deleted.",
        ),
    ] = None


class DaemonSetCondition(BaseModel):
    last_transition_time: Annotated[
        Optional[apimachinery.Time],
        Field(
            alias="lastTransitionTime",
            description="Last time the condition transitioned from one status to another.",
        ),
    ] = None
    message: Annotated[
        Optional[str],
        Field(description="A human readable message indicating details about the transition."),
    ] = None
    reason: Annotated[
        Optional[str],
        Field(description="The reason for the condition's last transition."),
    ] = None
    status: Annotated[
        str, Field(description="Status of the condition, one of True, False, Unknown.")
    ]
    type: Annotated[str, Field(description="Type of DaemonSet condition.")]


class DaemonSetStatus(BaseModel):
    collision_count: Annotated[
        Optional[int],
        Field(
            alias="collisionCount",
            description="Count of hash collisions for the DaemonSet. The DaemonSet controller uses this field as a collision avoidance mechanism when it needs to create the name for the newest ControllerRevision.",
        ),
    ] = None
    conditions: Annotated[
        Optional[List[DaemonSetCondition]],
        Field(
            description="Represents the latest available observations of a DaemonSet's current state."
        ),
    ] = None
    current_number_scheduled: Annotated[
        int,
        Field(
            alias="currentNumberScheduled",
            description="The number of nodes that are running at least 1 daemon pod and are supposed to run the daemon pod. More info: https://kubernetes.io/docs/concepts/workloads/controllers/daemonset/",
        ),
    ]
    desired_number_scheduled: Annotated[
        int,
        Field(
            alias="desiredNumberScheduled",
            description="The total number of nodes that should be running the daemon pod (including nodes correctly running the daemon pod). More info: https://kubernetes.io/docs/concepts/workloads/controllers/daemonset/",
        ),
    ]
    number_available: Annotated[
        Optional[int],
        Field(
            alias="numberAvailable",
            description="The number of nodes that should be running the daemon pod and have one or more of the daemon pod running and available (ready for at least spec.minReadySeconds)",
        ),
    ] = None
    number_misscheduled: Annotated[
        int,
        Field(
            alias="numberMisscheduled",
            description="The number of nodes that are running the daemon pod, but are not supposed to run the daemon pod. More info: https://kubernetes.io/docs/concepts/workloads/controllers/daemonset/",
        ),
    ]
    number_ready: Annotated[
        int,
        Field(
            alias="numberReady",
            description="numberReady is the number of nodes that should be running the daemon pod and have one or more of the daemon pod running with a Ready Condition.",
        ),
    ]
    number_unavailable: Annotated[
        Optional[int],
        Field(
            alias="numberUnavailable",
            description="The number of nodes that should be running the daemon pod and have none of the daemon pod running and available (ready for at least spec.minReadySeconds)",
        ),
    ] = None
    observed_generation: Annotated[
        Optional[int],
        Field(
            alias="observedGeneration",
            description="The most recent generation observed by the daemon set controller.",
        ),
    ] = None
    updated_number_scheduled: Annotated[
        Optional[int],
        Field(
            alias="updatedNumberScheduled",
            description="The total number of nodes that are running updated daemon pod",
        ),
    ] = None


class DeploymentCondition(BaseModel):
    last_transition_time: Annotated[
        Optional[apimachinery.Time],
        Field(
            alias="lastTransitionTime",
            description="Last time the condition transitioned from one status to another.",
        ),
    ] = None
    last_update_time: Annotated[
        Optional[apimachinery.Time],
        Field(
            alias="lastUpdateTime",
            description="The last time this condition was updated.",
        ),
    ] = None
    message: Annotated[
        Optional[str],
        Field(description="A human readable message indicating details about the transition."),
    ] = None
    reason: Annotated[
        Optional[str],
        Field(description="The reason for the condition's last transition."),
    ] = None
    status: Annotated[
        str, Field(description="Status of the condition, one of True, False, Unknown.")
    ]
    type: Annotated[str, Field(description="Type of deployment condition.")]


class DeploymentStatus(BaseModel):
    available_replicas: Annotated[
        Optional[int],
        Field(
            alias="availableReplicas",
            description="Total number of available pods (ready for at least minReadySeconds) targeted by this deployment.",
        ),
    ] = None
    collision_count: Annotated[
        Optional[int],
        Field(
            alias="collisionCount",
            description="Count of hash collisions for the Deployment. The Deployment controller uses this field as a collision avoidance mechanism when it needs to create the name for the newest ReplicaSet.",
        ),
    ] = None
    conditions: Annotated[
        Optional[List[DeploymentCondition]],
        Field(
            description="Represents the latest available observations of a deployment's current state."
        ),
    ] = None
    observed_generation: Annotated[
        Optional[int],
        Field(
            alias="observedGeneration",
            description="The generation observed by the deployment controller.",
        ),
    ] = None
    ready_replicas: Annotated[
        Optional[int],
        Field(
            alias="readyReplicas",
            description="readyReplicas is the number of pods targeted by this Deployment with a Ready Condition.",
        ),
    ] = None
    replicas: Annotated[
        Optional[int],
        Field(
            description="Total number of non-terminated pods targeted by this deployment (their labels match the selector)."
        ),
    ] = None
    unavailable_replicas: Annotated[
        Optional[int],
        Field(
            alias="unavailableReplicas",
            description="Total number of unavailable pods targeted by this deployment. This is the total number of pods that are still required for the deployment to have 100% available capacity. They may either be pods that are running but not yet available or pods that still have not been created.",
        ),
    ] = None
    updated_replicas: Annotated[
        Optional[int],
        Field(
            alias="updatedReplicas",
            description="Total number of non-terminated pods targeted by this deployment that have the desired template spec.",
        ),
    ] = None


class ReplicaSetCondition(BaseModel):
    last_transition_time: Annotated[
        Optional[apimachinery.Time],
        Field(
            alias="lastTransitionTime",
            description="The last time the condition transitioned from one status to another.",
        ),
    ] = None
    message: Annotated[
        Optional[str],
        Field(description="A human readable message indicating details about the transition."),
    ] = None
    reason: Annotated[
        Optional[str],
        Field(description="The reason for the condition's last transition."),
    ] = None
    status: Annotated[
        str, Field(description="Status of the condition, one of True, False, Unknown.")
    ]
    type: Annotated[str, Field(description="Type of replica set condition.")]


class ReplicaSetStatus(BaseModel):
    available_replicas: Annotated[
        Optional[int],
        Field(
            alias="availableReplicas",
            description="The number of available replicas (ready for at least minReadySeconds) for this replica set.",
        ),
    ] = None
    conditions: Annotated[
        Optional[List[ReplicaSetCondition]],
        Field(
            description="Represents the latest available observations of a replica set's current state."
        ),
    ] = None
    fully_labeled_replicas: Annotated[
        Optional[int],
        Field(
            alias="fullyLabeledReplicas",
            description="The number of pods that have labels matching the labels of the pod template of the replicaset.",
        ),
    ] = None
    observed_generation: Annotated[
        Optional[int],
        Field(
            alias="observedGeneration",
            description="ObservedGeneration reflects the generation of the most recently observed ReplicaSet.",
        ),
    ] = None
    ready_replicas: Annotated[
        Optional[int],
        Field(
            alias="readyReplicas",
            description="readyReplicas is the number of pods targeted by this ReplicaSet with a Ready Condition.",
        ),
    ] = None
    replicas: Annotated[
        int,
        Field(
            description="Replicas is the most recently observed number of replicas. More info: https://kubernetes.io/docs/concepts/workloads/controllers/replicationcontroller/#what-is-a-replicationcontroller"
        ),
    ]


class RollingUpdateDaemonSet(BaseModel):
    max_surge: Annotated[
        Optional[apimachinery.IntOrString],
        Field(
            alias="maxSurge",
            description="The maximum number of nodes with an existing available DaemonSet pod that can have an updated DaemonSet pod during during an update. Value can be an absolute number (ex: 5) or a percentage of desired pods (ex: 10%). This can not be 0 if MaxUnavailable is 0. Absolute number is calculated from percentage by rounding up to a minimum of 1. Default value is 0. Example: when this is set to 30%, at most 30% of the total number of nodes that should be running the daemon pod (i.e. status.desiredNumberScheduled) can have their a new pod created before the old pod is marked as deleted. The update starts by launching new pods on 30% of nodes. Once an updated pod is available (Ready for at least minReadySeconds) the old DaemonSet pod on that node is marked deleted. If the old pod becomes unavailable for any reason (Ready transitions to false, is evicted, or is drained) an updated pod is immediatedly created on that node without considering surge limits. Allowing surge implies the possibility that the resources consumed by the daemonset on any given node can double if the readiness check fails, and so resource intensive daemonsets should take into account that they may cause evictions during disruption.",
        ),
    ] = None
    max_unavailable: Annotated[
        Optional[apimachinery.IntOrString],
        Field(
            alias="maxUnavailable",
            description="The maximum number of DaemonSet pods that can be unavailable during the update. Value can be an absolute number (ex: 5) or a percentage of total number of DaemonSet pods at the start of the update (ex: 10%). Absolute number is calculated from percentage by rounding up. This cannot be 0 if MaxSurge is 0 Default value is 1. Example: when this is set to 30%, at most 30% of the total number of nodes that should be running the daemon pod (i.e. status.desiredNumberScheduled) can have their pods stopped for an update at any given time. The update starts by stopping at most 30% of those DaemonSet pods and then brings up new DaemonSet pods in their place. Once the new pods are available, it then proceeds onto other DaemonSet pods, thus ensuring that at least 70% of original number of DaemonSet pods are available at all times during the update.",
        ),
    ] = None


class RollingUpdateDeployment(BaseModel):
    max_surge: Annotated[
        Optional[apimachinery.IntOrString],
        Field(
            alias="maxSurge",
            description="The maximum number of pods that can be scheduled above the desired number of pods. Value can be an absolute number (ex: 5) or a percentage of desired pods (ex: 10%). This can not be 0 if MaxUnavailable is 0. Absolute number is calculated from percentage by rounding up. Defaults to 25%. Example: when this is set to 30%, the new ReplicaSet can be scaled up immediately when the rolling update starts, such that the total number of old and new pods do not exceed 130% of desired pods. Once old pods have been killed, new ReplicaSet can be scaled up further, ensuring that total number of pods running at any time during the update is at most 130% of desired pods.",
        ),
    ] = None
    max_unavailable: Annotated[
        Optional[apimachinery.IntOrString],
        Field(
            alias="maxUnavailable",
            description="The maximum number of pods that can be unavailable during the update. Value can be an absolute number (ex: 5) or a percentage of desired pods (ex: 10%). Absolute number is calculated from percentage by rounding down. This can not be 0 if MaxSurge is 0. Defaults to 25%. Example: when this is set to 30%, the old ReplicaSet can be scaled down to 70% of desired pods immediately when the rolling update starts. Once new pods are ready, old ReplicaSet can be scaled down further, followed by scaling up the new ReplicaSet, ensuring that the total number of pods available at all times during the update is at least 70% of desired pods.",
        ),
    ] = None


class RollingUpdateStatefulSetStrategy(BaseModel):
    max_unavailable: Annotated[
        Optional[apimachinery.IntOrString],
        Field(
            alias="maxUnavailable",
            description="The maximum number of pods that can be unavailable during the update. Value can be an absolute number (ex: 5) or a percentage of desired pods (ex: 10%). Absolute number is calculated from percentage by rounding up. This can not be 0. Defaults to 1. This field is alpha-level and is only honored by servers that enable the MaxUnavailableStatefulSet feature. The field applies to all pods in the range 0 to Replicas-1. That means if there is any unavailable pod in the range 0 to Replicas-1, it will be counted towards MaxUnavailable.",
        ),
    ] = None
    partition: Annotated[
        Optional[int],
        Field(
            description="Partition indicates the ordinal at which the StatefulSet should be partitioned for updates. During a rolling update, all pods from ordinal Replicas-1 to Partition are updated. All pods from ordinal Partition-1 to 0 remain untouched. This is helpful in being able to do a canary based deployment. The default value is 0."
        ),
    ] = None


class StatefulSetCondition(BaseModel):
    last_transition_time: Annotated[
        Optional[apimachinery.Time],
        Field(
            alias="lastTransitionTime",
            description="Last time the condition transitioned from one status to another.",
        ),
    ] = None
    message: Annotated[
        Optional[str],
        Field(description="A human readable message indicating details about the transition."),
    ] = None
    reason: Annotated[
        Optional[str],
        Field(description="The reason for the condition's last transition."),
    ] = None
    status: Annotated[
        str, Field(description="Status of the condition, one of True, False, Unknown.")
    ]
    type: Annotated[str, Field(description="Type of statefulset condition.")]


class StatefulSetStatus(BaseModel):
    available_replicas: Annotated[
        Optional[int],
        Field(
            alias="availableReplicas",
            description="Total number of available pods (ready for at least minReadySeconds) targeted by this statefulset.",
        ),
    ] = None
    collision_count: Annotated[
        Optional[int],
        Field(
            alias="collisionCount",
            description="collisionCount is the count of hash collisions for the StatefulSet. The StatefulSet controller uses this field as a collision avoidance mechanism when it needs to create the name for the newest ControllerRevision.",
        ),
    ] = None
    conditions: Annotated[
        Optional[List[StatefulSetCondition]],
        Field(
            description="Represents the latest available observations of a statefulset's current state."
        ),
    ] = None
    current_replicas: Annotated[
        Optional[int],
        Field(
            alias="currentReplicas",
            description="currentReplicas is the number of Pods created by the StatefulSet controller from the StatefulSet version indicated by currentRevision.",
        ),
    ] = None
    current_revision: Annotated[
        Optional[str],
        Field(
            alias="currentRevision",
            description="currentRevision, if not empty, indicates the version of the StatefulSet used to generate Pods in the sequence [0,currentReplicas).",
        ),
    ] = None
    observed_generation: Annotated[
        Optional[int],
        Field(
            alias="observedGeneration",
            description="observedGeneration is the most recent generation observed for this StatefulSet. It corresponds to the StatefulSet's generation, which is updated on mutation by the API Server.",
        ),
    ] = None
    ready_replicas: Annotated[
        Optional[int],
        Field(
            alias="readyReplicas",
            description="readyReplicas is the number of pods created for this StatefulSet with a Ready Condition.",
        ),
    ] = None
    replicas: Annotated[
        int,
        Field(description="replicas is the number of Pods created by the StatefulSet controller."),
    ]
    update_revision: Annotated[
        Optional[str],
        Field(
            alias="updateRevision",
            description="updateRevision, if not empty, indicates the version of the StatefulSet used to generate Pods in the sequence [replicas-updatedReplicas,replicas)",
        ),
    ] = None
    updated_replicas: Annotated[
        Optional[int],
        Field(
            alias="updatedReplicas",
            description="updatedReplicas is the number of Pods created by the StatefulSet controller from the StatefulSet version indicated by updateRevision.",
        ),
    ] = None


class StatefulSetUpdateStrategy(BaseModel):
    rolling_update: Annotated[
        Optional[RollingUpdateStatefulSetStrategy],
        Field(
            alias="rollingUpdate",
            description="RollingUpdate is used to communicate parameters when Type is RollingUpdateStatefulSetStrategyType.",
        ),
    ] = None
    type: Annotated[
        Optional[str],
        Field(
            description="Type indicates the type of the StatefulSetUpdateStrategy. Default is RollingUpdate."
        ),
    ] = None


class ControllerRevision(Resource):
    api_version: Annotated[
        Optional[Literal["apps/v1"]],
        Field(
            alias="apiVersion",
            description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
        ),
    ] = "apps/v1"
    data: Annotated[
        Optional[apimachinery.RawExtension],
        Field(description="Data is the serialized representation of the state."),
    ] = None
    kind: Annotated[
        Optional[Literal["ControllerRevision"]],
        Field(
            description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds"
        ),
    ] = "ControllerRevision"
    metadata: Annotated[
        Optional[apimachinery.ObjectMeta],
        Field(
            description="Standard object's metadata. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata"
        ),
    ] = None
    revision: Annotated[
        int,
        Field(description="Revision indicates the revision of the state represented by Data."),
    ]


ControllerRevisionList = ResourceList["ControllerRevision"]


class DaemonSetUpdateStrategy(BaseModel):
    rolling_update: Annotated[
        Optional[RollingUpdateDaemonSet],
        Field(
            alias="rollingUpdate",
            description='Rolling update config params. Present only if type = "RollingUpdate".',
        ),
    ] = None
    type: Annotated[
        Optional[str],
        Field(
            description='Type of daemon set update. Can be "RollingUpdate" or "OnDelete". Default is RollingUpdate.'
        ),
    ] = None


class DeploymentStrategy(BaseModel):
    rolling_update: Annotated[
        Optional[RollingUpdateDeployment],
        Field(
            alias="rollingUpdate",
            description="Rolling update config params. Present only if DeploymentStrategyType = RollingUpdate.",
        ),
    ] = None
    type: Annotated[
        Optional[str],
        Field(
            description='Type of deployment. Can be "Recreate" or "RollingUpdate". Default is RollingUpdate.'
        ),
    ] = None


class DaemonSetSpec(BaseModel):
    min_ready_seconds: Annotated[
        Optional[int],
        Field(
            alias="minReadySeconds",
            description="The minimum number of seconds for which a newly created DaemonSet pod should be ready without any of its container crashing, for it to be considered available. Defaults to 0 (pod will be considered available as soon as it is ready).",
        ),
    ] = None
    revision_history_limit: Annotated[
        Optional[int],
        Field(
            alias="revisionHistoryLimit",
            description="The number of old history to retain to allow rollback. This is a pointer to distinguish between explicit zero and not specified. Defaults to 10.",
        ),
    ] = None
    selector: Annotated[
        apimachinery.LabelSelector,
        Field(
            description="A label query over pods that are managed by the daemon set. Must match in order to be controlled. It must match the pod template's labels. More info: https://kubernetes.io/docs/concepts/overview/working-with-objects/labels/#label-selectors"
        ),
    ]
    template: Annotated[
        v1.PodTemplateSpec,
        Field(
            description='An object that describes the pod that will be created. The DaemonSet will create exactly one copy of this pod on every node that matches the template\'s node selector (or on every node if no node selector is specified). The only allowed template.spec.restartPolicy value is "Always". More info: https://kubernetes.io/docs/concepts/workloads/controllers/replicationcontroller#pod-template'
        ),
    ]
    update_strategy: Annotated[
        Optional[DaemonSetUpdateStrategy],
        Field(
            alias="updateStrategy",
            description="An update strategy to replace existing DaemonSet pods with new pods.",
        ),
    ] = None


class DeploymentSpec(BaseModel):
    min_ready_seconds: Annotated[
        Optional[int],
        Field(
            alias="minReadySeconds",
            description="Minimum number of seconds for which a newly created pod should be ready without any of its container crashing, for it to be considered available. Defaults to 0 (pod will be considered available as soon as it is ready)",
        ),
    ] = None
    paused: Annotated[
        Optional[bool], Field(description="Indicates that the deployment is paused.")
    ] = None
    progress_deadline_seconds: Annotated[
        Optional[int],
        Field(
            alias="progressDeadlineSeconds",
            description="The maximum time in seconds for a deployment to make progress before it is considered to be failed. The deployment controller will continue to process failed deployments and a condition with a ProgressDeadlineExceeded reason will be surfaced in the deployment status. Note that progress will not be estimated during the time a deployment is paused. Defaults to 600s.",
        ),
    ] = None
    replicas: Annotated[
        Optional[int],
        Field(
            description="Number of desired pods. This is a pointer to distinguish between explicit zero and not specified. Defaults to 1."
        ),
    ] = None
    revision_history_limit: Annotated[
        Optional[int],
        Field(
            alias="revisionHistoryLimit",
            description="The number of old ReplicaSets to retain to allow rollback. This is a pointer to distinguish between explicit zero and not specified. Defaults to 10.",
        ),
    ] = None
    selector: Annotated[
        apimachinery.LabelSelector,
        Field(
            description="Label selector for pods. Existing ReplicaSets whose pods are selected by this will be the ones affected by this deployment. It must match the pod template's labels."
        ),
    ]
    strategy: Annotated[
        Optional[DeploymentStrategy],
        Field(description="The deployment strategy to use to replace existing pods with new ones."),
    ] = None
    template: Annotated[
        v1.PodTemplateSpec,
        Field(
            description='Template describes the pods that will be created. The only allowed template.spec.restartPolicy value is "Always".'
        ),
    ]


class ReplicaSetSpec(BaseModel):
    min_ready_seconds: Annotated[
        Optional[int],
        Field(
            alias="minReadySeconds",
            description="Minimum number of seconds for which a newly created pod should be ready without any of its container crashing, for it to be considered available. Defaults to 0 (pod will be considered available as soon as it is ready)",
        ),
    ] = None
    replicas: Annotated[
        Optional[int],
        Field(
            description="Replicas is the number of desired replicas. This is a pointer to distinguish between explicit zero and unspecified. Defaults to 1. More info: https://kubernetes.io/docs/concepts/workloads/controllers/replicationcontroller/#what-is-a-replicationcontroller"
        ),
    ] = None
    selector: Annotated[
        apimachinery.LabelSelector,
        Field(
            description="Selector is a label query over pods that should match the replica count. Label keys and values that must match in order to be controlled by this replica set. It must match the pod template's labels. More info: https://kubernetes.io/docs/concepts/overview/working-with-objects/labels/#label-selectors"
        ),
    ]
    template: Annotated[
        Optional[v1.PodTemplateSpec],
        Field(
            description="Template is the object that describes the pod that will be created if insufficient replicas are detected. More info: https://kubernetes.io/docs/concepts/workloads/controllers/replicationcontroller#pod-template"
        ),
    ] = None


class StatefulSetSpec(BaseModel):
    min_ready_seconds: Annotated[
        Optional[int],
        Field(
            alias="minReadySeconds",
            description="Minimum number of seconds for which a newly created pod should be ready without any of its container crashing for it to be considered available. Defaults to 0 (pod will be considered available as soon as it is ready)",
        ),
    ] = None
    ordinals: Annotated[
        Optional[StatefulSetOrdinals],
        Field(
            description='ordinals controls the numbering of replica indices in a StatefulSet. The default ordinals behavior assigns a "0" index to the first replica and increments the index by one for each additional replica requested.'
        ),
    ] = None
    persistent_volume_claim_retention_policy: Annotated[
        Optional[StatefulSetPersistentVolumeClaimRetentionPolicy],
        Field(
            alias="persistentVolumeClaimRetentionPolicy",
            description="persistentVolumeClaimRetentionPolicy describes the lifecycle of persistent volume claims created from volumeClaimTemplates. By default, all persistent volume claims are created as needed and retained until manually deleted. This policy allows the lifecycle to be altered, for example by deleting persistent volume claims when their stateful set is deleted, or when their pod is scaled down. This requires the StatefulSetAutoDeletePVC feature gate to be enabled, which is beta.",
        ),
    ] = None
    pod_management_policy: Annotated[
        Optional[str],
        Field(
            alias="podManagementPolicy",
            description="podManagementPolicy controls how pods are created during initial scale up, when replacing pods on nodes, or when scaling down. The default policy is `OrderedReady`, where pods are created in increasing order (pod-0, then pod-1, etc) and the controller will wait until each pod is ready before continuing. When scaling down, the pods are removed in the opposite order. The alternative policy is `Parallel` which will create pods in parallel to match the desired scale without waiting, and on scale down will delete all pods at once.",
        ),
    ] = None
    replicas: Annotated[
        Optional[int],
        Field(
            description="replicas is the desired number of replicas of the given Template. These are replicas in the sense that they are instantiations of the same Template, but individual replicas also have a consistent identity. If unspecified, defaults to 1."
        ),
    ] = None
    revision_history_limit: Annotated[
        Optional[int],
        Field(
            alias="revisionHistoryLimit",
            description="revisionHistoryLimit is the maximum number of revisions that will be maintained in the StatefulSet's revision history. The revision history consists of all revisions not represented by a currently applied StatefulSetSpec version. The default value is 10.",
        ),
    ] = None
    selector: Annotated[
        apimachinery.LabelSelector,
        Field(
            description="selector is a label query over pods that should match the replica count. It must match the pod template's labels. More info: https://kubernetes.io/docs/concepts/overview/working-with-objects/labels/#label-selectors"
        ),
    ]
    service_name: Annotated[
        str,
        Field(
            alias="serviceName",
            description='serviceName is the name of the service that governs this StatefulSet. This service must exist before the StatefulSet, and is responsible for the network identity of the set. Pods get DNS/hostnames that follow the pattern: pod-specific-string.serviceName.default.svc.cluster.local where "pod-specific-string" is managed by the StatefulSet controller.',
        ),
    ]
    template: Annotated[
        v1.PodTemplateSpec,
        Field(
            description='template is the object that describes the pod that will be created if insufficient replicas are detected. Each pod stamped out by the StatefulSet will fulfill this Template, but have a unique identity from the rest of the StatefulSet. Each pod will be named with the format <statefulsetname>-<podindex>. For example, a pod in a StatefulSet named "web" with index number "3" would be named "web-3". The only allowed template.spec.restartPolicy value is "Always".'
        ),
    ]
    update_strategy: Annotated[
        Optional[StatefulSetUpdateStrategy],
        Field(
            alias="updateStrategy",
            description="updateStrategy indicates the StatefulSetUpdateStrategy that will be employed to update Pods in the StatefulSet when a revision is made to Template.",
        ),
    ] = None
    volume_claim_templates: Annotated[
        Optional[List[v1.PersistentVolumeClaim]],
        Field(
            alias="volumeClaimTemplates",
            description="volumeClaimTemplates is a list of claims that pods are allowed to reference. The StatefulSet controller is responsible for mapping network identities to claims in a way that maintains the identity of a pod. Every claim in this list must have at least one matching (by name) volumeMount in one container in the template. A claim in this list takes precedence over any volumes in the template, with the same name.",
        ),
    ] = None


class DaemonSet(Resource):
    api_version: Annotated[
        Optional[Literal["apps/v1"]],
        Field(
            alias="apiVersion",
            description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
        ),
    ] = "apps/v1"
    kind: Annotated[
        Optional[Literal["DaemonSet"]],
        Field(
            description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds"
        ),
    ] = "DaemonSet"
    metadata: Annotated[
        Optional[apimachinery.ObjectMeta],
        Field(
            description="Standard object's metadata. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata"
        ),
    ] = None
    spec: Annotated[
        Optional[DaemonSetSpec],
        Field(
            description="The desired behavior of this daemon set. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#spec-and-status"
        ),
    ] = None
    status: Annotated[
        Optional[DaemonSetStatus],
        Field(
            description="The current status of this daemon set. This data may be out of date by some window of time. Populated by the system. Read-only. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#spec-and-status"
        ),
    ] = None


DaemonSetList = ResourceList["DaemonSet"]


class Deployment(Resource):
    api_version: Annotated[
        Optional[Literal["apps/v1"]],
        Field(
            alias="apiVersion",
            description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
        ),
    ] = "apps/v1"
    kind: Annotated[
        Optional[Literal["Deployment"]],
        Field(
            description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds"
        ),
    ] = "Deployment"
    metadata: Annotated[
        Optional[apimachinery.ObjectMeta],
        Field(
            description="Standard object's metadata. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata"
        ),
    ] = None
    spec: Annotated[
        Optional[DeploymentSpec],
        Field(description="Specification of the desired behavior of the Deployment."),
    ] = None
    status: Annotated[
        Optional[DeploymentStatus],
        Field(description="Most recently observed status of the Deployment."),
    ] = None


DeploymentList = ResourceList["Deployment"]


class ReplicaSet(Resource):
    api_version: Annotated[
        Optional[Literal["apps/v1"]],
        Field(
            alias="apiVersion",
            description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
        ),
    ] = "apps/v1"
    kind: Annotated[
        Optional[Literal["ReplicaSet"]],
        Field(
            description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds"
        ),
    ] = "ReplicaSet"
    metadata: Annotated[
        Optional[apimachinery.ObjectMeta],
        Field(
            description="If the Labels of a ReplicaSet are empty, they are defaulted to be the same as the Pod(s) that the ReplicaSet manages. Standard object's metadata. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata"
        ),
    ] = None
    spec: Annotated[
        Optional[ReplicaSetSpec],
        Field(
            description="Spec defines the specification of the desired behavior of the ReplicaSet. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#spec-and-status"
        ),
    ] = None
    status: Annotated[
        Optional[ReplicaSetStatus],
        Field(
            description="Status is the most recently observed status of the ReplicaSet. This data may be out of date by some window of time. Populated by the system. Read-only. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#spec-and-status"
        ),
    ] = None


ReplicaSetList = ResourceList["ReplicaSet"]


class StatefulSet(Resource):
    api_version: Annotated[
        Optional[Literal["apps/v1"]],
        Field(
            alias="apiVersion",
            description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
        ),
    ] = "apps/v1"
    kind: Annotated[
        Optional[Literal["StatefulSet"]],
        Field(
            description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds"
        ),
    ] = "StatefulSet"
    metadata: Annotated[
        Optional[apimachinery.ObjectMeta],
        Field(
            description="Standard object's metadata. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata"
        ),
    ] = None
    spec: Annotated[
        Optional[StatefulSetSpec],
        Field(description="Spec defines the desired identities of pods in this set."),
    ] = None
    status: Annotated[
        Optional[StatefulSetStatus],
        Field(
            description="Status is the current status of Pods in this StatefulSet. This data may be out of date by some window of time."
        ),
    ] = None


StatefulSetList = ResourceList["StatefulSet"]
